# Copyright (c) 2013,Vienna University of Technology, Department of Geodesy and Geoinformation
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the <organization> nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


'''
Created on Jul 30, 2013

@author: Christoph Paulik christoph.paulik@geo.tuwien.ac.at
'''

import numpy as np

try:
    import pykdtree.kdtree as pykd
    pykdtree_installed = True
except ImportError:
    pykdtree_installed = False

try:
    import scipy.spatial as sc_spat
    scipy_installed = True
except ImportError:
    scipy_installed = False


class findGeoNN(object):

    """
    class that takes lat,lon coordinates, transformes them to cartesian (X,Y,Z)
    coordinates and provides a interface to scipy.spatial.kdTree
    as well as pykdtree if installed

    Parameters
    ----------
    lon : numpy.array or list
        longitudes of the points in the grid
    lat : numpy.array or list
        latitudes of the points in the grid
    R : float, optional
        Radius of the earth to use for calculating 3D coordinates
    grid : boolean, optional
        if True then lon and lat are assumed to be the coordinates of a grid
        and will be used in numpy.meshgrid to get coordinates for all
        grid points
    kd_tree_name : string, optional
        name of kdTree implementation to use, either
        'pykdtree' to use pykdtree or
        'scipy' to use scipy.spatial.kdTree
        Fallback is always scipy if any other string is given
        or if pykdtree is not installed. standard is pykdtree since it is faster

    Attributes
    ----------
    R : float
        earth radius to use in computation of x,y,z coordinates
    coords : numpy.array
        3D array of cartesian x,y,z coordinates
    kd_tree_name: string
        name of kdTree implementation to use, either
        'pykdtree' to use pykdtree or
        'scipy' to use scipy.spatial.kdTree
        Fallback is always scipy if any other string is given
        or if pykdtree is not installed
    kdtree: object
        kdTree object that is built only once and saved in this attribute

    Methods
    -------
    find_nearest_index(lon,lat)
        finds the nearest neighbor of the given lon,lat coordinates in the lon,lat
        arrays given during initialization and returns the index of the nearest neighbour
        in those arrays.

    """

    def __init__(self, lon, lat, R=6370997.0, grid=False, kd_tree_name='pykdtree'):
        """
        init method, prepares lon and lat arrays for _transform_lonlats if
        necessary

        """
        if grid:
            lon_grid, lat_grid = np.meshgrid(lon, lat)
            lat_init = lat_grid.flatten()
            lon_init = lon_grid.flatten()
            self.lat_size = len(lat)
            self.lon_size = len(lon)
        else:
            if lat.shape != lon.shape:
                raise Exception(
                    "lat and lon np.arrays have to have equal shapes")
            lat_init = lat
            lon_init = lon
        # Earth radius
        self.R = R
        self.kd_tree_name = kd_tree_name
        self.coords = self._transform_lonlats(lon_init, lat_init)
        self.kdtree = None
        self.grid = grid

    def _transform_lonlats(self, lon, lat):
        """
        calculates cartesian 3D coordinates from given lon,lat

        Parameters
        ----------
        lon : numpy.array, list or float
            longitudes of the points in the grid
        lat : numpy.array, list or float
            latitudes of the points in the grid

        Returns
        -------
        coords : np.array
            3D cartesian coordinates
        """
        lon = np.array(lon)
        lat = np.array(lat)
        coords = np.zeros((lon.size, 3))
        # calculated in float64, otherwise numerical inconsistencies happened
        # on different systems
        lons_rad = np.radians(lon, dtype=np.float64)
        lats_rad = np.radians(lat, dtype=np.float64)
        coords[:, 0] = self.R * np.cos(lats_rad) * np.cos(lons_rad)
        coords[:, 1] = self.R * np.cos(lats_rad) * np.sin(lons_rad)
        coords[:, 2] = self.R * np.sin(lats_rad)

        return coords

    def _build_kdtree(self):
        """
        Build the kdtree and saves it in the self.kdtree attribute
        """
        if self.kd_tree_name == 'pykdtree' and pykdtree_installed:
            self.kdtree = pykd.KDTree(self.coords)
        elif scipy_installed:
            self.kdtree = sc_spat.KDTree(self.coords)
        else:
            raise Exception("No supported kdtree implementation installed.\
                             Please install pykdtree or scipy.")

    def find_nearest_index(self, lon, lat, max_dist=np.Inf):
        """
        finds nearest index, builds kdTree if it does not yet exist

        Parameters
        ----------
        lon : float, list or numpy.array
            longitude of point
        lat : float, list or numpy.array
            latitude of point
        max_dist : float, optional
            maximum distance to consider for search

        Returns
        -------
        d : float, numpy.array
            distances of query coordinates to the nearest grid point,
            distance is given in cartesian coordinates and is not the
            great circle distance at the moment. This should be OK for
            most applications that look for the nearest neighbor which
            should not be hundreds of kilometers away.
        ind : int, numpy.array
            indices of nearest neighbor
        index_lon : numpy.array, optional
            if self.grid is True then return index into lon array of grid definition
        index_lat : numpy.array, optional
            if self.grid is True then return index into lat array of grid definition
        """
        if self.kdtree is None:
            self._build_kdtree()

        query_coords = self._transform_lonlats(lon, lat)

        d, ind = self.kdtree.query(query_coords, distance_upper_bound=max_dist)

        if not self.grid:
            return d, ind
        else:
            # calculate index position in grid definition arrays assuming row-major
            # flattening of arrays after numpy.meshgrid
            index_lat = ind / self.lon_size
            index_lon = ind % self.lon_size
            return d, index_lon.astype(np.int32), index_lat.astype(np.int32)



'''
Created on Mar 26, 2014

@author: Christoph Paulik christoph.paulik@geo.tuwien.ac.at
'''
import unittest
import numpy.testing as nptest
import numpy as np
from pygeogrids.grids import lonlat2cell
import pygeogrids as grids


class Test_lonlat2cell(unittest.TestCase):

    def setUp(self):
        lat = np.arange(-90, 90, 2.5)
        lon = np.arange(-180, 180, 2.5)
        self.lons, self.lats = np.meshgrid(lon, lat)

    def testlonlat2cell_hist(self):
        """
        setup grid with unequal cell size along lat and lon
        and test if the correct number of points lay in each cell
        """
        cells = lonlat2cell(
            self.lons, self.lats, cellsize_lon=15, cellsize_lat=30)
        hist, bin_edges = np.histogram(
            cells.flatten(), bins=len(np.unique(cells)))
        nptest.assert_allclose(hist, np.zeros_like(hist) + 72)


class TestFindNearestNeighbor(unittest.TestCase):

    def setUp(self):
        self.grid = grids.genreg_grid(1, 1)

    def test_nearest_neighbor(self):
        gpi, dist = self.grid.find_nearest_gpi(14.3, 18.5)
        assert gpi == 25754
        assert len([dist]) == 1
        lon, lat = self.grid.gpi2lonlat(gpi)
        assert lon == 14.5
        assert lat == 18.5

    def test_nearest_neighbor_list(self):
        gpi, dist = self.grid.find_nearest_gpi([145.1, 90.2], [45.8, -16.3])
        assert len(gpi) == 2
        assert len(dist) == 2
        assert gpi[0] == 16165
        assert gpi[1] == 38430
        lon, lat = self.grid.gpi2lonlat(gpi)
        assert lon[0] == 145.5
        assert lon[1] == 90.5
        assert lat[0] == 45.5
        assert lat[1] == -16.5

    def test_nearest_neighbor_ndarray(self):
        gpi, dist = self.grid.find_nearest_gpi(
            np.array([145.1, 90.2]), np.array([45.8, -16.3]))
        assert len(gpi) == 2
        assert len(dist) == 2
        assert gpi[0] == 16165
        assert gpi[1] == 38430
        lon, lat = self.grid.gpi2lonlat(gpi)
        assert lon[0] == 145.5
        assert lon[1] == 90.5
        assert lat[0] == 45.5
        assert lat[1] == -16.5

    def test_nearest_neighbor_numpy_single(self):
        gpi, dist = self.grid.find_nearest_gpi(
            np.array([145.1, 90.2])[0], np.array([45.8, -16.3])[0])
        assert gpi == 16165
        lon, lat = self.grid.gpi2lonlat(gpi)
        assert lon == 145.5
        assert lat == 45.5


class TestCellGrid(unittest.TestCase):

    """
    setup simple 2D grid 2.5 degree global grid (144x72)
    which starts at the North Western corner of 90 -180
    Test for cell specific features
    """

    def setUp(self):
        self.latdim = np.arange(90, -90, -2.5)
        self.londim = np.arange(-180, 180, 2.5)
        self.lon, self.lat = np.meshgrid(self.londim, self.latdim)
        self.grid = grids.BasicGrid(
            self.lon.flatten(), self.lat.flatten(), shape=(len(self.londim), len(self.latdim)))
        self.cellgrid = self.grid.to_cell_grid()

    def test_gpi2cell(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = 200
        cell = self.cellgrid.gpi2cell(gpi)
        assert cell == 1043

    def test_gpi2cell_iterable(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = [200, 255]
        cell = self.cellgrid.gpi2cell(gpi)
        assert np.all(cell == [1043, 2015])

    def test_gpi2cell_numpy_single(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = np.array([200, 255])[0]
        cell = self.cellgrid.gpi2cell(gpi)
        assert cell == 1043

    def test_gpi2cell_custom_gpis(self):
        """
        test if gpi to row column lookup works correctly
        """
        self.custom_gpi_grid = grids.BasicGrid(self.lon.flatten(),
                                               self.lat.flatten(),
                                               shape=(len(self.londim),
                                                      len(self.latdim)),
                                               gpis=np.arange(len(self.lat.flatten()))[::-1])
        self.custom_gpi_cell_grid = self.custom_gpi_grid.to_cell_grid()
        gpi = [200, 255]
        cell = self.custom_gpi_cell_grid.gpi2cell(gpi)
        assert np.all(cell == [1549, 577])
        gpi = 200
        cell = self.custom_gpi_cell_grid.gpi2cell(gpi)
        assert cell == 1549


class Test_2Dgrid(unittest.TestCase):

    """
    setup simple 2D grid 2.5 degree global grid (144x72)
    which starts at the North Western corner of 90 -180
    and test 2D lookup
    """

    def setUp(self):
        self.latdim = np.arange(90, -90, -2.5)
        self.londim = np.arange(-180, 180, 2.5)
        self.lon, self.lat = np.meshgrid(self.londim, self.latdim)
        self.grid = grids.BasicGrid(
            self.lon.flatten(), self.lat.flatten(), shape=(len(self.londim), len(self.latdim)))

    def test_gpi2rowcol(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = 200
        row_should = 1
        column_should = 200 - 144
        row, column = self.grid.gpi2rowcol(gpi)
        assert row == row_should
        assert column == column_should

    def test_gpi2rowcol_np_int(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = np.array([200])[0]
        row_should = 1
        column_should = 200 - 144
        row, column = self.grid.gpi2rowcol(gpi)
        assert row == row_should
        assert column == column_should

    def test_gpi2rowcol_iterable(self):
        """
        test if gpi to row column lookup works correctly
        """
        gpi = [143, 200, 255]
        row_should = [0, 1, 1]
        column_should = [143, 200 - 144, 255 - 144]
        row, column = self.grid.gpi2rowcol(gpi)
        assert np.all(row == row_should)
        assert np.all(column == column_should)

    def test_gpi2rowcol_custom_gpis(self):
        """
        test if gpi to row column lookup works correctly
        """
        self.custom_gpi_grid = grids.BasicGrid(self.lon.flatten(),
                                               self.lat.flatten(),
                                               shape=(len(self.londim),
                                                      len(self.latdim)),
                                               gpis=np.arange(len(self.lat.flatten()))[::-1])
        gpi = [200, 255]
        row_should = [70, 70]
        column_should = [87, 32]
        row, column = self.custom_gpi_grid.gpi2rowcol(gpi)
        assert np.all(row == row_should)
        assert np.all(column == column_should)

    def test_gpi2lonlat(self):
        """
        test if gpi to longitude latitude lookup works correctly
        """
        gpi = 200
        lat_should = 87.5
        lon_should = -180 + (200 - 144) * 2.5
        lon, lat = self.grid.gpi2lonlat(gpi)
        assert lon == lon_should
        assert lat == lat_should

    def test_tocellgrid(self):
        """
        test if to_cell_grid method works correctly
        """
        cell_grid = self.grid.to_cell_grid()
        result = grids.BasicGrid.__eq__(self.grid, cell_grid)
        assert result


def test_genreggrid():
    """
    test generation of regular grids
    """
    grid = grids.genreg_grid()
    assert grid.shape == (360, 180)
    lon, lat = grid.gpi2lonlat(3)
    assert lon == -176.5
    assert lat == 89.5


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



# Copyright (c) 2013,Vienna University of Technology, Department of Geodesy and Geoinformation
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Vienna University of Technology, Department of Geodesy and Geoinformation nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
Created on Jan 21, 2014

Module for saving grid to netCDF

@author: Christoph Paulik christoph.paulik@geo.tuwien.ac.at
'''
from netCDF4 import Dataset
import numpy as np
import os
from datetime import datetime

from pygeogrids import CellGrid, BasicGrid


def save_lonlat(filename, arrlon, arrlat, arrcell=None,
                gpis=None, subsets={}, global_attrs=None):
    """
    saves grid information to netCDF file

    Parameters
    ----------
    filename : string
        name of file
    arrlon : numpy.array
        array of longitudes
    arrlat : numpy.array
        array of latitudes
    arrcell : numpy.array, optional
        array of cell numbers
    gpis : numpy.array, optional
        gpi numbers if not index of arrlon, arrlat
    subsets : dict of dicts, optional
        keys : long_name of the netcdf variables
        values : dict with the following keys: points, meaning
        e.g. subsets = {'subset_flag': {'points': numpy.array,
                                        'meaning': 'water, land'}}
    global_attrs : dict, optional
        if given will be written as global attributs into netCDF file
    """

    nc_name = filename

    with Dataset(nc_name, 'w', format='NETCDF4') as ncfile:

        if (global_attrs is not None and 'shape' in global_attrs and
                type(global_attrs['shape']) is not int and
                len(global_attrs['shape']) == 2):

            latsize = global_attrs['shape'][1]
            lonsize = global_attrs['shape'][0]
            ncfile.createDimension("lat", latsize)
            ncfile.createDimension("lon", lonsize)
            arrlat = np.unique(arrlat)[::-1]  # sorts arrlat descending
            arrlon = np.unique(arrlon)

            gpisize = global_attrs['shape'][0] * global_attrs['shape'][1]
            if gpis is None:
                gpivalues = np.arange(gpisize,
                                      dtype=np.int32).reshape(latsize,
                                                              lonsize)
                gpivalues = gpivalues[::-1]
            else:
                gpivalues = gpis.reshape(latsize, lonsize)
        else:
            ncfile.createDimension("gp", arrlon.size)
            gpisize = arrlon.size
            if gpis is None:
                gpivalues = np.arange(arrlon.size, dtype=np.int32)
            else:
                gpivalues = gpis

        dim = list(ncfile.dimensions.keys())

        gpi = ncfile.createVariable('gpi', np.dtype('int32').char, dim)

        if gpis is None:
            gpi[:] = gpivalues
            setattr(gpi, 'long_name', 'Grid point index')
            setattr(gpi, 'units', '')
            setattr(gpi, 'valid_range', [0, gpisize])
            gpidirect = 0x1b
        else:
            gpi[:] = gpivalues
            setattr(gpi, 'long_name', 'Grid point index')
            setattr(gpi, 'units', '')
            setattr(gpi, 'valid_range', [np.min(gpivalues), np.max(gpivalues)])
            gpidirect = 0x0b

        latitude = ncfile.createVariable('lat', np.dtype('float64').char,
                                         dim[0])
        latitude[:] = arrlat
        setattr(latitude, 'long_name', 'Latitude')
        setattr(latitude, 'units', 'degree_north')
        setattr(latitude, 'standard_name', 'latitude')
        setattr(latitude, 'valid_range', [-90.0, 90.0])

        if len(dim) == 2:
            londim = dim[1]
        else:
            londim = dim[0]
        longitude = ncfile.createVariable('lon', np.dtype('float64').char,
                                          londim)
        longitude[:] = arrlon
        setattr(longitude, 'long_name', 'Longitude')
        setattr(longitude, 'units', 'degree_east')
        setattr(longitude, 'standard_name', 'longitude')
        setattr(longitude, 'valid_range', [-180.0, 180.0])

        if arrcell is not None:
            cell = ncfile.createVariable('cell', np.dtype('int16').char, dim)
            cell[:] = arrcell
            setattr(cell, 'long_name', 'Cell')
            setattr(cell, 'units', '')
            setattr(cell, 'valid_range', [np.min(arrcell), np.max(arrcell)])

        if subsets:
            for subset_name in subsets.keys():
                flag = ncfile.createVariable(subset_name, np.dtype('int8').char,
                                             dim)

                # create flag array based on shape of data
                lf = np.zeros_like(gpivalues)
                if len(dim) == 2:
                    lf = lf.flatten()
                lf[subsets[subset_name]['points']] = 1
                if len(dim) == 2:
                    lf = lf.reshape(latsize, lonsize)

                flag[:] = lf
                setattr(flag, 'long_name', subset_name)
                setattr(flag, 'units', '')
                setattr(flag, 'coordinates', 'lat lon')
                setattr(flag, 'flag_values', np.arange(2, dtype=np.int8))
                setattr(flag, 'flag_meanings', subsets[subset_name]['meaning'])
                setattr(flag, 'valid_range', [0, 1])

        s = "%Y-%m-%d %H:%M:%S"
        date_created = datetime.now().strftime(s)

        attr = {'Conventions': 'CF-1.6',
                'id': os.path.split(filename)[1],  # file name
                'date_created': date_created,
                'geospatial_lat_min': np.round(np.min(arrlat), 4),
                'geospatial_lat_max': np.round(np.max(arrlat), 4),
                'geospatial_lon_min': np.round(np.min(arrlon), 4),
                'geospatial_lon_max': np.round(np.max(arrlon), 4),
                'gpidirect': gpidirect
                }

        ncfile.setncatts(attr)
        if global_attrs is not None and type(global_attrs) is dict:
            ncfile.setncatts(global_attrs)


def save_grid(filename, grid, subset_name='subset_flag',
              subset_meaning="water, land", global_attrs=None):
    """
    save a BasicGrid or CellGrid to netCDF
    it is assumed that a subset should be used as land_points

    Parameters
    ----------
    filename : string
        name of file
    grid : BasicGrid or CellGrid object
        grid whose definition to save to netCDF
    subset_name : string, optional
        long_name of the netcdf variable
        if the subset symbolises something other than a land/sea mask
    subset_meaning : string, optional
        will be written into flag_meanings metadata of variable 'subset_name'
    global_attrs : dict, optional
        if given will be written as global attributs into netCDF file
    """

    try:
        arrcell = grid.arrcell
    except AttributeError:
        arrcell = None

    if grid.gpidirect is True:
        gpis = None
    else:
        gpis = grid.gpis

    if grid.shape is not None:
        if global_attrs is None:
            global_attrs = {}
        global_attrs['shape'] = grid.shape

    subsets = {subset_name: {'points': grid.subset, 'meaning': subset_meaning}}

    save_lonlat(filename, grid.arrlon, grid.arrlat, arrcell=arrcell,
                gpis=gpis, subsets=subsets, global_attrs=global_attrs)


def load_grid(filename, subset_flag='subset_flag'):
    """
    load a grid from netCDF file

    Parameters
    ----------
    filename : string
        filename
    subset_flag : string, optional
        name of the subset to load.

    Returns
    -------
    grid : BasicGrid or CellGrid instance
        grid instance initialized with the loaded data
    """

    with Dataset(filename, 'r') as nc_data:
        # determine if it is a cell grid or a basic grid
        arrcell = None
        if 'cell' in nc_data.variables.keys():
            arrcell = nc_data.variables['cell'][:]

        # determine if gpis are in order or custom order
        if nc_data.gpidirect == 0x1b:
            gpis = None  # gpis can be calculated through np.arange..
        else:
            gpis = nc_data.variables['gpi'][:]

        shape = None
        if hasattr(nc_data, 'shape'):
            try:
                shape = tuple(nc_data.shape)
            except TypeError as e:
                try:
                    length = len(nc_data.shape)
                except TypeError:
                    length = nc_data.shape.size
                if length == 1:
                    shape = tuple([nc_data.shape])
                else:
                    raise e

        subset = None
        # some old grid do not have a shape attribute
        # this meant that they had shape of len 1
        if shape is None:
            shape = tuple([len(nc_data.variables['lon'][:])])

        # check if grid has regular shape
        if len(shape) == 2:
            lons, lats = np.meshgrid(nc_data.variables['lon'][:],
                                     nc_data.variables['lat'][::-1])
            lons = lons.flatten('F')
            lats = lats.flatten('F')

            if subset_flag in nc_data.variables.keys():
                subset = np.where(
                    nc_data.variables[subset_flag][:].flatten() == 1)[0]

        elif len(shape) == 1:
            lons = nc_data.variables['lon'][:]
            lats = nc_data.variables['lat'][:]

            # determine if it has a subset
            if subset_flag in nc_data.variables.keys():
                subset = np.where(nc_data.variables[subset_flag][:] == 1)[0]

        if arrcell is None:
            # BasicGrid
            return BasicGrid(lons,
                             lats,
                             gpis=gpis,
                             subset=subset,
                             shape=shape)
        else:
            # CellGrid
            return CellGrid(nc_data.variables['lon'][:],
                            nc_data.variables['lat'][:],
                            arrcell,
                            gpis=gpis,
                            subset=subset,
                            shape=shape)




# This file was generated by PyScaffold from the git
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '0.1.3'
version_full = '668a064a916785631e724f74f9a509816410d31f'
def get_versions(default=dict(), verbose=False):
    return dict(version=version_version, full=version_full)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    Setup file for pygeogrids.

    This file was generated with PyScaffold 2.0.4, a tool that easily
    puts up a scaffold for your new Python project. Learn more under:
    http://pyscaffold.readthedocs.org/
"""

import inspect
import os
import re
import subprocess
import sys
from contextlib import contextmanager
from distutils.cmd import Command
from distutils.command.build import build as _build
from distutils.command.sdist import sdist as _sdist
from glob import glob

import setuptools
from setuptools import setup
from setuptools.command.test import test as TestCommand

# For Python 2/3 compatibility, pity we can't use six.moves here
try:  # try Python 3 imports first
    import configparser
    from io import StringIO
except ImportError:  # then fall back to Python 2
    import ConfigParser as configparser
    from StringIO import StringIO

__location__ = os.path.join(os.getcwd(), os.path.dirname(
    inspect.getfile(inspect.currentframe())))

# general settings
pyscaffold_version = "2.0.4"
package = "pygeogrids"
namespace = []
root_pkg = namespace[0] if namespace else package
if namespace:
    pkg_path = os.path.join(*namespace[-1].split('.') + [package])
else:
    pkg_path = package

# Version configuration
versionfile_path = os.path.join(pkg_path, '_version.py')
tag_prefix = 'v'  # tags are like v1.2.0
short_version_py = """
# This file was generated by PyScaffold from the git
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

version_version = '{version}'
version_full = '{full}'
def get_versions(default=dict(), verbose=False):
    return dict(version=version_version, full=version_full)
"""


class PyTest(TestCommand):
    user_options = [("cov=", None, "Run coverage"),
                    ("cov-report=", None, "Generate a coverage report"),
                    ("junitxml=", None, "Generate xml of test results")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.cov = None
        self.cov_report = None
        self.junitxml = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        if self.cov:
            self.cov = ["--cov", self.cov, "--cov-report", "term-missing"]
            if self.cov_report:
                self.cov.extend(["--cov-report", self.cov_report])
        if self.junitxml:
            self.junitxml = ["--junitxml", self.junitxml]

    def run_tests(self):
        try:
            import pytest
        except:
            raise RuntimeError("py.test is not installed, "
                               "run: pip install pytest")
        params = {"args": self.test_args}
        if self.cov:
            params["args"] += self.cov
            params["plugins"] = ["cov"]
        if self.junitxml:
            params["args"] += self.junitxml
        errno = pytest.main(**params)
        sys.exit(errno)


def sphinx_builder():
    try:
        from sphinx.setup_command import BuildDoc
    except ImportError:
        class NoSphinx(Command):
            user_options = []

            def initialize_options(self):
                raise RuntimeError("Sphinx documentation is not installed, "
                                   "run: pip install sphinx")

        return NoSphinx

    class BuildSphinxDocs(BuildDoc):

        def run(self):
            if self.builder == "doctest":
                import sphinx.ext.doctest as doctest
                # Capture the DocTestBuilder class in order to return the total
                # number of failures when exiting
                ref = capture_objs(doctest.DocTestBuilder)
                BuildDoc.run(self)
                errno = ref[-1].total_failures
                sys.exit(errno)
            else:
                BuildDoc.run(self)

    return BuildSphinxDocs


class ObjKeeper(type):
    instances = {}

    def __init__(cls, name, bases, dct):
        cls.instances[cls] = []

    def __call__(cls, *args, **kwargs):
        cls.instances[cls].append(super(ObjKeeper, cls).__call__(*args,
                                                                 **kwargs))
        return cls.instances[cls][-1]


def capture_objs(cls):
    from six import add_metaclass
    module = inspect.getmodule(cls)
    name = cls.__name__
    keeper_class = add_metaclass(ObjKeeper)(cls)
    setattr(module, name, keeper_class)
    cls = getattr(module, name)
    return keeper_class.instances[cls]


def get_install_requirements(path):
    content = open(os.path.join(__location__, path)).read()
    return [req for req in content.splitlines() if req != '']


def read(fname):
    with open(os.path.join(__location__, fname)) as fh:
        content = fh.read()
    # StringIO seems to expect a unicode object under Windows 7
    if sys.version_info[0] < 3:
        content = content.decode(encoding='utf8')
    return content


def str2bool(val):
    return val.lower() in ("yes", "true")


def read_setup_cfg():
    config = configparser.SafeConfigParser(allow_no_value=True)
    config_file = StringIO(read(os.path.join(__location__, 'setup.cfg')))
    config.readfp(config_file)
    metadata = dict(config.items('metadata'))
    classifiers = metadata.get('classifiers', '')
    metadata['classifiers'] = [item.strip() for item in classifiers.split(',')]
    console_scripts = dict(config.items('console_scripts'))
    console_scripts = prepare_console_scripts(console_scripts)
    package_data = metadata.get('package_data', '')
    package_data = [item.strip() for item in package_data.split(',') if item]
    metadata['package_data'] = package_data
    data_files = metadata.get('data_files', '')
    data_files = [item.strip() for item in data_files.split(',')]
    data_files = [item for pattern in data_files for item in glob(pattern)]
    metadata['data_files'] = data_files
    return metadata, console_scripts


def prepare_console_scripts(dct):
    return ['{cmd} = {func}'.format(cmd=k, func=v) for k, v in dct.items()]


@contextmanager
def stash(filename):
    with open(filename) as fh:
        old_content = fh.read()
    try:
        yield
    finally:
        with open(filename, 'w') as fh:
            fh.write(old_content)


class ShellCommand(object):

    def __init__(self, command, shell=True, cwd=None):
        self._command = command
        self._shell = shell
        self._cwd = cwd

    def __call__(self, *args):
        command = "{cmd} {args}".format(cmd=self._command,
                                        args=subprocess.list2cmdline(args))
        output = subprocess.check_output(command,
                                         shell=self._shell,
                                         cwd=self._cwd,
                                         stderr=subprocess.STDOUT,
                                         universal_newlines=True)
        return self._yield_output(output)

    def _yield_output(self, msg):
        for line in msg.splitlines():
            yield line


def get_git_cmd(**args):
    if sys.platform == "win32":
        for cmd in ["git.cmd", "git.exe"]:
            git = ShellCommand(cmd, **args)
            try:
                git("--version")
            except (subprocess.CalledProcessError, OSError):
                continue
            return git
        return None
    else:
        git = ShellCommand("git", **args)
        try:
            git("--version")
        except (subprocess.CalledProcessError, OSError):
            return None
        return git


def version_from_git(tag_prefix, root, verbose=False):
    # this runs 'git' from the root of the source tree. This only gets called
    # if the git-archive 'subst' keywords were *not* expanded, and
    # _version.py hasn't already been rewritten with a short version string,
    # meaning we're inside a checked out source tree.
    git = get_git_cmd(cwd=root)
    if not git:
        if verbose:
            print("no git found")
        return None
    tag = next(git("describe", "--tags", "--dirty", "--always"))
    if not tag.startswith(tag_prefix):
        if verbose:
            print("tag '{}' doesn't start with prefix '{}'".format(tag,
                                                                   tag_prefix))
        return None
    tag = tag[len(tag_prefix):]
    sha1 = next(git("rev-parse", "HEAD"))
    full = sha1.strip()
    if tag.endswith("-dirty"):
        full += "-dirty"
    return {"version": tag, "full": full}


def get_keywords(versionfile_abs):
    # the code embedded in _version.py can just fetch the value of these
    # keywords. When used from setup.py, we don't want to import _version.py,
    # so we do it with a regexp instead. This function is not used from
    # _version.py.
    keywords = dict()
    try:
        with open(versionfile_abs, "r") as fh:
            for line in fh.readlines():
                if line.strip().startswith("git_refnames ="):
                    mo = re.search(r'=\s*"(.*)"', line)
                    if mo:
                        keywords["refnames"] = mo.group(1)
                if line.strip().startswith("git_full ="):
                    mo = re.search(r'=\s*"(.*)"', line)
                    if mo:
                        keywords["full"] = mo.group(1)
    except EnvironmentError:
        return None
    return keywords


def version_from_keywords(keywords, tag_prefix, verbose=False):
    if not keywords:
        return None  # keyword-finding function failed to find keywords
    refnames = keywords["refnames"].strip()
    if refnames.startswith("$Format"):
        if verbose:
            print("keywords are unexpanded, not using")
        return None  # unexpanded, so not in an unpacked git-archive tarball
    refs = set([r.strip() for r in refnames.strip("()").split(",")])
    # starting in git-1.8.3, tags are listed as "tag: foo-1.0" instead of
    # just "foo-1.0". If we see a "tag: " prefix, prefer those.
    TAG = "tag: "
    tags = set([r[len(TAG):] for r in refs if r.startswith(TAG)])
    if not tags:
        # Either we're using git < 1.8.3, or there really are no tags. We use
        # a heuristic: assume all version tags have a digit. The old git %d
        # expansion behaves like git log --decorate=short and strips out the
        # refs/heads/ and refs/tags/ prefixes that would let us distinguish
        # between branches and tags. By ignoring refnames without digits, we
        # filter out many common branch names like "release" and
        # "stabilization", as well as "HEAD" and "master".
        tags = set([r for r in refs if re.search(r'\d', r)])
        if verbose:
            print("discarding '%s', no digits" % ",".join(refs - tags))
    if verbose:
        print("likely tags: %s" % ",".join(sorted(tags)))
    for ref in sorted(tags):
        # sorting will prefer e.g. "2.0" over "2.0rc1"
        if ref.startswith(tag_prefix):
            r = ref[len(tag_prefix):]
            if verbose:
                print("picking %s" % r)
            return {"version": r,
                    "full": keywords["full"].strip()}
    else:
        if verbose:
            print("no suitable tags, using full revision id")
        return {"version": keywords["full"].strip(),
                "full": keywords["full"].strip()}


def version_from_file(filename):
    versions = dict()
    try:
        with open(filename) as fh:
            for line in fh.readlines():
                mo = re.match("version_version = '([^']+)'", line)
                if mo:
                    versions["version"] = mo.group(1)
                mo = re.match("version_full = '([^']+)'", line)
                if mo:
                    versions["full"] = mo.group(1)
    except EnvironmentError:
        return None

    return versions


def version_from_parentdir(parentdir_prefix, root, verbose=False):
    # Source tarballs conventionally unpack into a directory that includes
    # both the project name and a version string.
    dirname = os.path.basename(root)
    if not dirname.startswith(parentdir_prefix):
        if verbose:
            print("guessing rootdir is '%s', but '%s' doesn't start with "
                  "prefix '%s'" % (root, dirname, parentdir_prefix))
        return None
    version = dirname[len(parentdir_prefix):].split('-')[0]
    return {"version": version, "full": ""}


def git2pep440(ver_str):
    dash_count = ver_str.count('-')
    if dash_count == 0:
        return ver_str
    elif dash_count == 1:
        return ver_str.split('-')[0] + "+dirty"
    elif dash_count == 2:
        tag, commits, sha1 = ver_str.split('-')
        return "{}.post0.dev{}+{}".format(tag, commits, sha1)
    elif dash_count == 3:
        tag, commits, sha1, _ = ver_str.split('-')
        return "{}.post0.dev{}+{}.dirty".format(tag, commits, sha1)
    else:
        raise RuntimeError("Invalid version string")


def get_versions(verbose=False):
    versionfile_abs = os.path.join(__location__, versionfile_path)
    vcs_kwds = get_keywords(versionfile_abs)
    parentdir = package + '-'
    root = __location__

    # different version retrieval methods as (method, args, comment)
    ver_retrieval = [
        (version_from_keywords, (vcs_kwds, tag_prefix, verbose),
         "expanded keywords"),
        (version_from_file, (versionfile_abs,), "file"),
        (version_from_git, (tag_prefix, root, verbose), "git"),
        (version_from_parentdir, (parentdir, root, verbose), "parentdir")
    ]
    for method, args, comment in ver_retrieval:
        ver = method(*args)
        if ver:
            if verbose:
                print("got version from {}".format(comment))
            break
    else:
        ver = {"version": "unknown", "full": "unknown"}
    ver['version'] = git2pep440(ver['version'])
    return ver


class cmd_version(Command):
    description = "report generated version string"
    user_options = []
    boolean_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        ver = get_versions(verbose=True)["version"]
        print("Version: %s" % ver)


class cmd_build(_build):

    def run(self):
        versions = get_versions(verbose=True)
        _build.run(self)
        # now locate _version.py in the new build/ directory and replace it
        # with an updated value
        target_versionfile = os.path.join(self.build_lib,
                                          versionfile_path)
        print("updating {}...".format(target_versionfile))
        os.unlink(target_versionfile)
        with open(target_versionfile, "w") as fh:
            fh.write(short_version_py.format(**versions))


if 'cx_Freeze' in sys.modules:  # cx_freeze enabled?
    from cx_Freeze.dist import build_exe as _build_exe

    class cmd_build_exe(_build_exe):

        def run(self):
            versions = get_versions(verbose=True)
            with stash(versionfile_path):
                print("updating {}...".format(versionfile_path))
                with open(versionfile_path, 'w') as fh:
                    fh.write(short_version_py.format(**versions))
                _build_exe.run(self)


class cmd_sdist(_sdist):

    def run(self):
        versions = get_versions(verbose=True)
        self._generated_versions = versions
        # unless we update this, the command will keep using the old version
        self.distribution.metadata.version = versions["version"]
        return _sdist.run(self)

    def make_release_tree(self, base_dir, files):
        _sdist.make_release_tree(self, base_dir, files)
        # now locate _version.py in the new base_dir directory (remembering
        # that it may be a hardlink) and replace it with an updated value
        target_versionfile = os.path.join(base_dir, versionfile_path)
        print("updating {}...".format(target_versionfile))
        os.unlink(target_versionfile)
        with open(target_versionfile, "w") as fh:
            fh.write(short_version_py.format(**self._generated_versions))


def setup_package():
    # Assemble additional setup commands
    cmdclass = dict(docs=sphinx_builder(),
                    doctest=sphinx_builder(),
                    test=PyTest,
                    version=cmd_version,
                    sdist=cmd_sdist,
                    build=cmd_build)
    if 'cx_Freeze' in sys.modules:  # cx_freeze enabled?
        cmdclass['build_exe'] = cmd_build_exe
        del cmdclass['build']

    # Some helper variables
    version = get_versions()["version"]
    docs_path = os.path.join(__location__, "docs")
    docs_build_path = os.path.join(docs_path, "_build")
    install_reqs = get_install_requirements("requirements.txt")
    metadata, console_scripts = read_setup_cfg()

    command_options = {
        'docs': {'project': ('setup.py', package),
                 'version': ('setup.py', version.split('-', 1)[0]),
                 'release': ('setup.py', version),
                 'build_dir': ('setup.py', docs_build_path),
                 'config_dir': ('setup.py', docs_path),
                 'source_dir': ('setup.py', docs_path)},
        'doctest': {'project': ('setup.py', package),
                    'version': ('setup.py', version.split('-', 1)[0]),
                    'release': ('setup.py', version),
                    'build_dir': ('setup.py', docs_build_path),
                    'config_dir': ('setup.py', docs_path),
                    'source_dir': ('setup.py', docs_path),
                    'builder': ('setup.py', 'doctest')},
        'test': {'test_suite': ('setup.py', 'tests'),
                 'cov': ('setup.py', root_pkg)}}

    setup(name=package,
          version=version,
          url=metadata['url'],
          description=metadata['description'],
          author=metadata['author'],
          author_email=metadata['author_email'],
          license=metadata['license'],
          long_description=read('README.md'),
          classifiers=metadata['classifiers'],
          test_suite='tests',
          packages=setuptools.find_packages(exclude=['tests', 'tests.*']),
          namespace_packages=namespace,
          install_requires=install_reqs,
          setup_requires=['six'],
          cmdclass=cmdclass,
          tests_require=['pytest-cov', 'pytest'],
          package_data={package: metadata['package_data']},
          data_files=[('.', metadata['data_files'])],
          command_options=command_options,
          entry_points={'console_scripts': console_scripts},
          zip_safe=False)  # do not zip egg file after setup.py install


if __name__ == "__main__":
    setup_package()



# Copyright (c) 2013,Vienna University of Technology, Department of Geodesy and Geoinformation
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Vienna University of Technology, Department of Geodesy and Geoinformation nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'''
Created on Aug 26, 2013

@author: Christoph Paulik Christoph.Paulik@geo.tuwien.ac.at
'''

import pygeogrids.nearest_neighbor as NN

import numpy as np
import numpy.testing as nptest
try:
    from itertools import izip as zip
except ImportError:
    pass  # python3


class GridDefinitionError(Exception):
    pass


class GridIterationError(Exception):
    pass


class BasicGrid(object):

    """
    Grid that just has lat,lon coordinates and can find the
    nearest neighbour. It can also yield the gpi, lat, lon
    information in order.

    Parameters
    ----------
    lon : numpy.array
        longitudes of the points in the grid
    lat : numpy.array
        latitudes of the points in the grid
    gpis : numpy.array, optional
        if the gpi numbers are in a different order than the
        lon and lat arrays an array containing the gpi numbers
        can be given
        if no array is given here the lon lat arrays are given
        gpi numbers starting at 0
    subset : numpy.array, optional
        if the active part of the array is only a subset of
        all the points then the subset array which is a index
        into lon and lat can be given here
    setup_kdTree : boolean, optional
        if set (default) then the kdTree for nearest neighbour
        search will be built on initialization
    shape : tuple, optional
        The shape of the grid array in 2-d space.
        e.g. for a 1x1 degree global regular grid the shape would be (360,180).
        if given the grid can be reshaped into the given shape
        this indicates that it is a regular grid and fills the
        attributes self.lon2d and self.lat2d which
        define the grid only be the meridian coordinates(self.lon2d) and
        the coordinates of the circles of latitude(self.lat2d).
        The shape has to be given as (lon2d, lat2d)
        It it is not given the shape is set to the length of the input
        lon and lat arrays.

    Attributes
    ----------
    arrlon : numpy.array
        1D array of all longitudes of the grid
    arrlat : numpy.array
        1D array of all latitudes of the grid
    n_gpi : int
        number of gpis in the grid
    gpidirect : boolean
        if true the gpi number is equal to the index
        of arrlon and arrlat
    gpis : numpy.array
        gpi number for elements in arrlon and arrlat
        gpi[i] is located at arrlon[i],arrlat[i]
    subset : numpy.array
        if given then this contains the indices of a subset of
        the grid. This can be used if only a part of a grid is
        interesting for a application. e.g. land points, or only
        a specific country
    allpoints : boolean
        if False only a subset of the grid is active
    activearrlon : numpy.array
        array of longitudes that are active, is defined by
        arrlon[subset] if a subset is given otherwise equal to
        arrlon
    activearrlat : numpy.array
        array of latitudes that are active, is defined by
        arrlat[subset] if a subset is given otherwise equal to
        arrlat
    activegpis : numpy.array
        array of gpis that are active, is defined by
        gpis[subset] if a subset is given otherwise equal to
        gpis
    issplit : boolean
        if True then the array was split in n parts with
        the self.split function
    kdTree : object
        grid.nearest_neighbor.findGeoNN object for
        nearest neighbor search
    shape : tuple, optional
        if given during initialization then this is
        the shape the grid can be reshaped to
    lat2d : numpy.array, optional
        if shape is given this attribute contains all latitudes according to
        the provided 2d-shape that make up the grid
    lon2d : numpy.array, optional
        if shape is given this attribute contains all longitudes according to
        the provided 2d-shape that make up the grid
    """

    def __init__(self, lon, lat, gpis=None, subset=None, setup_kdTree=True,
                 shape=None):
        """
        init method, prepares lon and lat arrays for _transform_lonlats if
        necessary

        """
        if lat.shape != lon.shape:
            raise GridDefinitionError(
                "lat and lon np.arrays have to have equal shapes")

        self.n_gpi = len(lon)

        self.arrlon = lon
        self.arrlat = lat

        self.shape = None

        if shape is not None and len(shape) == 2:
            if len(self.arrlat) % shape[0] != 0:
                raise GridDefinitionError("Given shape does not have the correct first dimension."
                                          " Length of lat array is not divisible by shape[0] without rest")

            if len(self.arrlon) % shape[1] != 0:
                raise GridDefinitionError("Given shape does not have the correct second dimension."
                                          " Length of lon array is not divisible by shape[1] without rest")

            self.shape = shape
            self.lat2d = np.reshape(self.arrlat, self.shape)
            self.lon2d = np.reshape(self.arrlon, self.shape)

        else:
            self.shape = tuple([len(self.arrlon)])

        if gpis is None:
            self.gpis = np.arange(self.n_gpi)
            self.gpidirect = True
        else:
            if lat.shape != gpis.shape:
                raise GridDefinitionError(
                    "lat, lon gpi np.arrays have to have equal shapes")
            self.gpis = gpis
            self.gpidirect = False

        self.subset = subset

        if subset is not None:
            self.activearrlon = self.arrlon[subset]
            self.activearrlat = self.arrlat[subset]
            self.activegpis = self.gpis[subset]
            self.allpoints = False
        else:
            self.activearrlon = self.arrlon
            self.activearrlat = self.arrlat
            self.activegpis = self.gpis
            self.allpoints = True

        self.issplit = False

        self.kdTree = None
        if setup_kdTree:
            self._setup_kdtree()

    def _setup_kdtree(self):
        """
        setup kdTree
        """
        if self.kdTree is None:
            self.kdTree = NN.findGeoNN(self.activearrlon, self.activearrlat)
            self.kdTree._build_kdtree()

    def split(self, n):
        """function splits the grid into n parts
        this changes not function but grid_points() which takes
        the argument n and will only iterate through this part of
        the grid

        Parameters
        ----------
        n : int
            number of parts the grid should be split into
        """
        self.issplit = True
        self.subarrlats = np.array_split(self.activearrlat, n)
        self.subarrlons = np.array_split(self.activearrlon, n)
        self.subgpis = np.array_split(self.activegpis, n)

    def unite(self):
        """
        unites a split array, so that it can be iterated over as a whole
        again
        """
        self.issplit = False

    def grid_points(self, *args):
        """
        Yields all grid points in order

        Parameters
        ----------
        n : int, optional
            if the grid is split in n parts using the split function
            then this iterator will only iterate of the nth part of the
            grid

        Returns
        -------
        gpi : long
            grid point index
        lon : float
            longitude of gpi
        lat : float
            longitude of gpi
        """

        if not self.issplit and len(args) == 0:
            return self._normal_grid_points()
        elif self.issplit and len(args) == 1:
            return self._split_grid_points(args[0])

        raise GridIterationError("this function only takes an argument if the grid is split, "
                                 "and takes no argument if the grid is not split")

    def get_grid_points(self, *args):
        """
        Returns all active grid points

        Parameters
        ----------
        n : int, optional
            if the grid is split in n parts using the split function
            then this function will only return the nth part of the
            grid

        Returns
        -------
        gpis : numpy.array
        arrlon : numpy.array
        arrlat :numpy.array
        """

        if not self.issplit and len(args) == 0:
            return (self.activegpis,
                    self.activearrlon,
                    self.activearrlat)

        elif self.issplit and len(args) == 1:
            n = args[0]
            return (self.subgpis[n],
                    self.subarrlons[n],
                    self.subarrlats[n])

    def _normal_grid_points(self):
        """
        Yields all grid points in order

        Returns
        -------
        gpi : long
            grid point index
        lon : float
            longitude of gpi
        lat : float
            longitude of gpi
        """

        for i, (lon, lat) in enumerate(zip(self.activearrlon, self.activearrlat)):
            yield self.activegpis[i], lon, lat

    def _split_grid_points(self, n):
        """
        Yields all grid points or split grid in order

        Parameters
        ----------
        n : int
            number of subgrid to yield

        Returns
        -------
        gpi : long
            grid point index
        lon : float
            longitude of gpi
        lat : float
            longitude of gpi
        """

        for i, (lon, lat) in enumerate(zip(self.subarrlons[n], self.subarrlats[n])):
            yield self.subgpis[n][i], lon, lat

    def find_nearest_gpi(self, lon, lat, max_dist=np.Inf):
        """
        finds nearest gpi, builds kdTree if it does not yet exist

        Parameters
        ----------
        lon : float or iterable
            longitude of point
        lat : float or iterable
            latitude of point

        Returns
        -------
        gpi : long
            grid point index
        distance : float
            distance of gpi to given lon, lat
            At the moment not on a great circle but in spherical cartesian coordinates
        """
        # check if input is iterable
        iterable = _element_iterable(lon)

        if self.kdTree is None:
            self._setup_kdtree()

        d, ind = self.kdTree.find_nearest_index(lon, lat, max_dist=max_dist)

        if not iterable:
            d = d[0]
            ind = ind[0]

        if self.gpidirect and self.allpoints:
            return ind, d

        return self.activegpis[ind], d

    def gpi2lonlat(self, gpi):
        """
        Longitude and Latitude for given GPI.

        Parameters
        ----------
        gpi : int32 or iterable
            Grid Point Index.

        Returns
        -------
        lon : float
            Longitude (deg) of GPI.
        lat : float
        """
        if self.gpidirect:
            return self.arrlon[gpi], self.arrlat[gpi]
        else:
            index = np.where(self.activegpis == gpi)[0][0]
            return self.activearrlon[index], self.activearrlat[index]

    def gpi2rowcol(self, gpi):
        """
        if the grid can be reshaped into a
        sensible 2D shape then this function gives
        the row(latitude dimension) and column(longitude dimension)
        indices of the gpi in the 2D grid

        Parameters
        ----------
        gpi : int32
            Grid Point Index.

        Returns
        -------
        row : int
            row in 2D array
        col : int
            column in 2D array
        """
        # check if iterable
        iterable = _element_iterable(gpi)

        gpi = np.atleast_1d(gpi)
        if len(self.shape) == 2:
            if self.gpidirect:
                index = gpi
            else:
                # get the indices that would sort the gpis
                gpisorted = np.argsort(self.gpis)
                # find the position where the gpis fit in the sorted array
                pos = np.searchsorted(self.gpis[gpisorted], gpi)
                index = gpisorted[pos]

            index_lat = (index / self.shape[0]).astype(np.int)
            index_lon = index % self.shape[0]
            if not iterable:
                index_lat = index_lat[0]
                index_lon = index_lon[0]
            return index_lat, index_lon

        else:
            raise(GridDefinitionError("Grid has no 2D shape"))

    def calc_lut(self, other, max_dist=np.Inf, into_subset=False):
        """
        takes other BasicGrid or CellGrid objects and computes
        a lookup table between them.
        the lut will have the size of self.n_gpis and will
        for every grid point have the nearest index into other.arrlon etc.

        Parameters
        ----------
        other : grid object
            to which to calculate the lut to
        max_dist : float, optional
            maximum allowed distance in meters
        into_subset : boolean, optional
            if set the returned lut will have the index into the subset
            if the other grid is a subset of a grid.
            Example:
            if e.g. ind_l is used for the warp_grid some datasets will
            be given as arrays with len(ind_l) elements. These
            datasets can not be indexed with gpi numbers but have to
            be indexed with indices into the subset
        """

        if self.kdTree is None:
            self._setup_kdtree()

        if other.kdTree is None:
            other._setup_kdtree()

        if self.kdTree.kdtree is not None and other.kdTree.kdtree is not None:
            dist, index = other.kdTree.find_nearest_index(
                self.activearrlon, self.activearrlat, max_dist=max_dist)

            valid_index = np.where(dist != np.inf)[0]
            dist = dist[valid_index]
            index = index[valid_index]
            if not other.gpidirect or not other.allpoints:
                if not into_subset:
                    index = other.activegpis[index]

            active_lut = np.empty_like(self.activearrlat, dtype=np.int64)
            active_lut.fill(-1)
            active_lut[valid_index] = index

            if not self.allpoints:
                gpi_lut = np.empty_like(self.gpis)
                gpi_lut.fill(-1)
                gpi_lut[self.subset] = active_lut
            else:
                gpi_lut = active_lut

            return gpi_lut

    def get_bbox_grid_points(self, latmin=-90, latmax=90, lonmin=-180, lonmax=180, coords=False):
        """
        Returns all grid points located in a submitted geographic box,
        optinal as coordinates

        Parameters
        ----------
        latmin : float, optional
            minimum latitude
        latmax : float, optional
            maximum latitude
        lonmin : float, optional
            minimum latitude
        lonmax : float, optional
            maximum latitude
        coords : boolean, optional
            set to True if coordinates should be returned

        Returns
        -------
        gpi : numpy.array
            grid point indices, if coords=False
        lat : numpy.array
            longitudes of gpis, if coords=True
        lon : numpy.array
            longitudes of gpis, if coords=True
        """

        index = np.where((self.activearrlat <= latmax) &
                         (self.activearrlat >= latmin) &
                         (self.activearrlon <= lonmax) &
                         (self.activearrlon >= lonmin))

        if coords is True:
            return self.activearrlat[index], self.activearrlon[index]
        else:
            return self.activegpis[index]

    def to_cell_grid(self, cellsize=5.0, cellsize_lat=None, cellsize_lon=None):
        """
        convert grid to cellgrid with a cell partition of
        cellsize

        Parameters
        ----------
        cellsize : float, optional
            cell size in degrees
        cellsize_lon : float, optional
            Cell size in degrees on the longitude axis
        cellsize_lat : float, optional
            Cell size in degrees on the latitude axis

        Returns
        -------
        cell_grid : CellGrid object
            cell grid object
        """
        cells = lonlat2cell(self.arrlon, self.arrlat, cellsize=cellsize,
                            cellsize_lat=cellsize_lat,
                            cellsize_lon=cellsize_lon)

        if self.gpidirect:
            gpis = None
        else:
            gpis = self.gpis
        return CellGrid(self.arrlon, self.arrlat, cells, gpis=gpis, subset=self.subset,
                        shape=self.shape)

    def __eq__(self, other):
        """
        compare arrlon, arrlat, gpis, subsets and shape

        Returns
        -------
        result : boolean
        """
        # only test to certain significance for float variables
        try:
            nptest.assert_allclose(self.arrlon, other.arrlon)
            lonsame = True
        except AssertionError:
            lonsame = False
        try:
            nptest.assert_allclose(self.arrlat, other.arrlat)
            latsame = True
        except AssertionError:
            latsame = False
        gpisame = np.all(self.gpis == other.gpis)
        if self.subset is not None and other.subset is not None:
            subsetsame = np.all(self.subset == other.subset)
        elif self.subset is None and other.subset is None:
            subsetsame = True
        else:
            subsetsame = False

        if self.shape is None and other.shape is None:
            shapesame = True
        elif self.shape is not None and other.shape is not None:
            shapesame = self.shape == other.shape
        else:
            shapesame = False

        return np.all([lonsame, latsame, gpisame, subsetsame, shapesame])


class CellGrid(BasicGrid):

    """
    Grid that has lat,lon coordinates as well as cell informatin.
    It can find nearest neighbour. It can also yield the gpi, lat, lon, cell
    information in cell order. This is important if the data on the grid
    is saved in cell files on disk as we can go through all grid points
    with optimized IO performance

    Parameters
    ----------
    lon : numpy.array
        longitudes of the points in the grid
    lat : numpy.array
        latitudes of the points in the grid
    cells : numpy.array
        of same shape as lon and lat, containing the cell number
        of each gpi
    gpis : numpy.array, optional
        if the gpi numbers are in a different order than the
        lon and lat arrays an array containing the gpi numbers
        can be given
    subset : numpy.array, optional
        if the active part of the array is only a subset of
        all the points then the subset array which is a index
        into lon, lat and cells can be given here

    Attributes
    ----------
    arrcell : numpy.array
        array of cell number with same shape as arrlon,arrlat
    activearrcell : numpy.array
        array of longitudes that are active,
        is defined by arrlon[subset] if a subset is given otherwise equal to arrlon
    """

    def __init__(self, lon, lat, cells, gpis=None, subset=None, setup_kdTree=False, **kwargs):
        super(CellGrid, self).__init__(lon, lat, gpis=gpis, subset=subset,
                                       setup_kdTree=setup_kdTree, **kwargs)

        if self.arrlon.shape != cells.shape:
            raise GridDefinitionError(
                "lat, lon and cells np.arrays have to have equal shapes")
        self.arrcell = cells

        if subset is not None:
            self.activearrcell = self.arrcell[subset]
        else:
            self.activearrcell = self.arrcell

    def gpi2cell(self, gpi):
        """
        Cell for given GPI.

        Parameters
        ----------
        gpi : int32 or iterable
            Grid Point Index.

        Returns
        -------
        cell : int or iterable
            Cell number of GPI.
        """
        # check if iterable
        iterable = _element_iterable(gpi)

        gpi = np.atleast_1d(gpi)
        if self.gpidirect:
            cell = self.arrcell[gpi]
        else:
            # get the indices that would sort the gpis
            gpisorted = np.argsort(self.gpis)
            # find the position where the gpis fit in the sorted array
            pos = np.searchsorted(self.gpis[gpisorted], gpi)
            index = gpisorted[pos]
            cell = self.activearrcell[index]

        if not iterable:
            cell = cell[0]

        return cell

    def get_cells(self):
        """
        function to get all cell numbers of the grid

        Returns
        -------
        cells : numpy.array
            unique cell numbers
        """
        return np.unique(self.activearrcell)

    def get_grid_points(self, *args):
        """
        Returns all active grid points

        Parameters
        ----------
        n : int, optional
            if the grid is split in n parts using the split function
            then this function will only return the nth part of the
            grid

        Returns
        -------
        gpis : numpy.array
        arrlon : numpy.array
        arrlat :numpy.array
        cells : numpy.array
        """

        if not self.issplit and len(args) == 0:
            return (self.activegpis,
                    self.activearrlon,
                    self.activearrlat,
                    self.activearrcell)

        elif self.issplit and len(args) == 1:
            n = args[0]
            return (self.subgpis[n],
                    self.subarrlons[n],
                    self.subarrlats[n],
                    self.subcells[n])

    def grid_points_for_cell(self, cell):
        """
        get all grid points for a given cell number

        Parameters
        ----------
        cell : int
            cell number

        Returns
        -------
        gpis : numpy.array
            gpis belonging to cell
        lons : numpy.array
            longitudes belonging to the gpis
        lats : numpy.array
            latitudes belonging to the gpis
        """
        cell_index = np.where(cell == self.activearrcell)

        return (self.activegpis[cell_index],
                self.activearrlon[cell_index],
                self.activearrlat[cell_index])

    def split(self, n):
        """function splits the grid into n parts
        this changes not function but grid_points() which takes
        the argument n and will only iterate through this part of
        the grid

        Parameters
        ----------
        n : int
            number of parts the grid should be split into
        """
        self.issplit = True
        # sort by cell number to split correctly
        sorted_index = np.argsort(self.activearrcell)
        self.subarrlats = np.array_split(self.activearrlat[sorted_index], n)
        self.subarrlons = np.array_split(self.activearrlon[sorted_index], n)
        self.subgpis = np.array_split(self.activegpis[sorted_index], n)
        self.subcells = np.array_split(self.activearrcell[sorted_index], n)

    def _normal_grid_points(self):
        """
        Yields all grid points in cell order

        Returns
        -------
        gpi : long
            grid point index
        lon : float
            longitude of gpi
        lat : float
            longitude of gpi
        cell : int
            cell number
        """

        uniq_cells = np.unique(self.activearrcell)

        for cell in uniq_cells:
            cell_gpis = np.where(cell == self.activearrcell)[0]
            for gpi in cell_gpis:
                yield self.activegpis[gpi], self.activearrlon[gpi], self.activearrlat[gpi], cell

    def _split_grid_points(self, n):
        """
        Yields all grid points in cell order

        Parameters
        ----------
        n : int
                number of subgrid to yield

        Returns
        -------
        gpi : long
            grid point index
        lon : float
            longitude of gpi
        lat : float
            longitude of gpi
        cell : int
            cell number
        """

        uniq_cells = np.unique(self.subcells[n])

        for cell in uniq_cells:
            cell_gpis = np.where(cell == self.subcells[n])[0]
            for gpi in cell_gpis:
                yield self.subgpis[n][gpi], self.subarrlons[n][gpi], self.subarrlats[n][gpi], cell

    def __eq__(self, other):
        """
        compare arcells

        Returns
        -------
        result : boolean
        """
        basicsame = super(CellGrid, self).__eq__(other)
        cellsame = np.all(self.arrcell == other.arrcell)
        return np.all([basicsame, cellsame])


def lonlat2cell(lon, lat, cellsize=5., cellsize_lon=None, cellsize_lat=None):
    """
    Partition lon, lat points into cells.

    Parameters
    ----------
    lat: float64, or numpy.array
        Latitude.
    lon: float64, or numpy.array
        Longitude.
    cellsize: float
        Cell size in degrees
    cellsize_lon : float, optional
        Cell size in degrees on the longitude axis
    cellsize_lat : float, optional
        Cell size in degrees on the latitude axis

    Returns
    -------
    cell: int32, or numpy.array
        Cell.
    """

    if cellsize_lon is None:
        cellsize_lon = cellsize
    if cellsize_lat is None:
        cellsize_lat = cellsize
    y = np.clip(np.floor((np.double(lat) +
                          (np.double(90.0) + 1e-9)) / cellsize_lat), 0, 180)
    x = np.clip(
        np.floor((np.double(lon) + (np.double(180.0) + 1e-9)) / cellsize_lon), 0, 360)
    return np.int32(x * (np.double(180.0) / cellsize_lat) + y)


def gridfromdims(londim, latdim):
    """
    Defines new grid object from latitude and longitude dimensions.
    latitude and longitude dimensions are 1D arrays that give the
    latitude and longitude values of a 2D latitude-longitude array

    Parameters
    ----------
    londim: numpy.array
        longitude dimension
    latdim: numpy.array
        latitude dimension

    """
    lons, lats = np.meshgrid(londim, latdim)
    return BasicGrid(lons.flatten(), lats.flatten(),
                     shape=(len(londim), len(latdim)))


def genreg_grid(grd_spc_lat=1, grd_spc_lon=1,
                minlat=-90.0, maxlat=90.0,
                minlon=-180.0, maxlon=180.0):
    """
    Define a global regular lon lat grid which starts in the North Western
    Corner of minlon, maxlat.
    The grid points are defined to be in the middle of a grid cell.
    e.g. the first point on a 1x1 degree grid with
    minlon -180.0 and maxlat 90.0 will be at
    -179.5 longitude, 89.5 latitude

    Parameters
    ----------
    grd_spc_lat: float, optional
        grid spacing in latitude direction
    grd_spc_lon: float, optional
        grid spacing in longitude direction
    minlat : float, optional
        minimum latitude of the grid
    maxlat : float, optional
        maximum latitude of the grid
    minlon : float, optional
        minimum longitude of the grid
    maxlon : float, optional
        maximum longitude of the grid
    """
    lon_dim = np.arange(minlon + grd_spc_lon / 2.0, maxlon, grd_spc_lon)
    lat_dim = np.arange(maxlat - grd_spc_lat / 2.0, minlat, -grd_spc_lat)

    return gridfromdims(lon_dim, lat_dim)


def _element_iterable(el):
    """
    Test if a element is iterable

    Parameters
    ----------
    el: object


    Returns
    -------
    iterable: boolean
       if True then then el is iterable
       if Fales then not
    """
    try:
        el[0]
        iterable = True
    except (TypeError, IndexError):
        iterable = False

    return iterable



# Copyright (c) 2013,Vienna University of Technology, Department of Geodesy and Geoinformation
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the Vienna University of Technology, Department of Geodesy and Geoinformation nor the
#      names of its contributors may be used to endorse or promote products
#      derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL VIENNA UNIVERSITY OF TECHNOLOGY,
# DEPARTMENT OF GEODESY AND GEOINFORMATION BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
Created on Jan 21, 2014

@author: Christoph Paulik christoph.paulik@geo.tuwien.ac.at
'''
import unittest
import numpy as np
import numpy.testing as nptest
from netCDF4 import Dataset

import pygeogrids.netcdf as grid_nc
import pygeogrids as grids
import tempfile


class Test(unittest.TestCase):

    def setUp(self):
        lat, lon = np.arange(180) - 90, np.arange(360) - 180
        self.lats, self.lons = np.meshgrid(lat, lon)
        self.lats, self.lons = self.lats.flatten(), self.lons.flatten()
        self.cells = grids.lonlat2cell(self.lons, self.lats)
        self.subset = np.sort(np.random.choice(np.arange(self.lats.size),
                                               size=500, replace=False))
        self.basic = grids.BasicGrid(self.lons, self.lats, subset=self.subset,
                                     shape=(360, 180))
        self.basic_generated = grids.genreg_grid(1, 1)
        self.basic_irregular = grids.BasicGrid(np.random.random(360 * 180) * 360 - 180,
                                               np.random.random(
                                                   360 * 180) * 180 - 90,
                                               subset=self.subset)
        self.cellgrid = grids.CellGrid(self.lons, self.lats, self.cells,
                                       subset=self.subset)

        self.testfile = tempfile.NamedTemporaryFile().name

    def tearDown(self):
        pass

    def test_save_lonlat_nc(self):
        grid_nc.save_lonlat(self.testfile,
                            self.lons, self.lats, self.cells,
                            subsets={'subset_flag': {'points': self.subset,
                                                     'meaning': 'test_flag'}},
                            global_attrs={'test': 'test_attribute'})

        with Dataset(self.testfile) as nc_data:
            nptest.assert_array_equal(self.lats, nc_data.variables['lat'][:])
            nptest.assert_array_equal(self.lons, nc_data.variables['lon'][:])
            nptest.assert_array_equal(self.cells, nc_data.variables['cell'][:])
            nptest.assert_array_equal(
                self.subset, np.where(nc_data.variables['subset_flag'][:] == 1)[0])
            assert nc_data.test == 'test_attribute'

    def test_save_basicgrid_nc(self):
        grid_nc.save_grid(self.testfile,
                          self.basic,
                          global_attrs={'test': 'test_attribute'})

        with Dataset(self.testfile) as nc_data:
            nptest.assert_array_equal(np.unique(self.lats)[::-1],
                                      nc_data.variables['lat'][:])
            nptest.assert_array_equal(np.unique(self.lons),
                                      nc_data.variables['lon'][:])

            nptest.assert_array_equal(self.subset,
                                      np.where(nc_data.variables['subset_flag'][:].flatten() == 1)[0])
            assert nc_data.test == 'test_attribute'
            assert nc_data.shape[1] == 180
            assert nc_data.shape[0] == 360

    def test_save_basicgrid_generated(self):
        grid_nc.save_grid(self.testfile,
                          self.basic,
                          global_attrs={'test': 'test_attribute'})

        with Dataset(self.testfile) as nc_data:
            nptest.assert_array_equal(np.unique(self.lats)[::-1],
                                      nc_data.variables['lat'][:])
            nptest.assert_array_equal(np.unique(self.lons),
                                      nc_data.variables['lon'][:])

            nptest.assert_array_equal(self.subset,
                                      np.where(nc_data.variables['subset_flag'][:].flatten() == 1)[0])
            assert nc_data.test == 'test_attribute'
            assert nc_data.shape[1] == 180
            assert nc_data.shape[0] == 360

    def test_save_basicgrid_irregular_nc(self):
        grid_nc.save_grid(self.testfile,
                          self.basic_irregular,
                          global_attrs={'test': 'test_attribute'})

        with Dataset(self.testfile) as nc_data:
            nptest.assert_array_equal(
                self.basic_irregular.arrlat, nc_data.variables['lat'][:])
            nptest.assert_array_equal(
                self.basic_irregular.arrlon, nc_data.variables['lon'][:])
            nptest.assert_array_equal(self.subset,
                                      np.where(nc_data.variables['subset_flag'][:] == 1)[0])
            assert nc_data.test == 'test_attribute'
            assert nc_data.shape == 64800

    def test_save_cellgrid_nc(self):
        grid_nc.save_grid(self.testfile,
                          self.cellgrid,
                          global_attrs={'test': 'test_attribute'})

        with Dataset(self.testfile) as nc_data:
            nptest.assert_array_equal(self.lats, nc_data.variables['lat'][:])
            nptest.assert_array_equal(self.lons, nc_data.variables['lon'][:])
            nptest.assert_array_equal(self.cells, nc_data.variables['cell'][:])
            nptest.assert_array_equal(
                self.subset, np.where(nc_data.variables['subset_flag'][:] == 1)[0])
            assert nc_data.test == 'test_attribute'
            assert nc_data.gpidirect == 0x1b

    def test_save_load_basicgrid(self):
        grid_nc.save_grid(self.testfile,
                          self.basic)

        loaded_grid = grid_nc.load_grid(self.testfile)
        assert self.basic == loaded_grid

    def test_save_load_basicgrid_irregular(self):
        grid_nc.save_grid(self.testfile,
                          self.basic_irregular)

        loaded_grid = grid_nc.load_grid(self.testfile)
        assert self.basic_irregular == loaded_grid

    def test_save_load_cellgrid(self):
        grid_nc.save_grid(self.testfile,
                          self.cellgrid)

        loaded_grid = grid_nc.load_grid(self.testfile)
        assert self.cellgrid == loaded_grid

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
